@@ -3,7 +3,9 @@
 import sys
 import requests
 import threading
+import sqlite3
 
+from ..cache import insert_into_table
 from ..console import console
 from ..errors import ParsedNoneError, call_on_error
 from ..settings import OP, DICTS
@@ -21,57 +23,44 @@ CAMBRIDGE_URL = "https://dictionary.cambridge.org"
 CAMBRIDGE_DICT_BASE_URL = CAMBRIDGE_URL + "/dictionary/english/"
 CAMBRIDGE_SPELLCHECK_URL = CAMBRIDGE_URL + "/spellcheck/english/?q="
 
+CAMBRIDGE_DICT_BASE_URL_CN = CAMBRIDGE_URL + "/dictionary/english-chinese-simplified/"
+CAMBRIDGE_SPELLCHECK_URL_CN = CAMBRIDGE_URL + "/spellcheck/english-chinese-simplified/?q="
+# CAMBRIDGE_DICT_BASE_URL_CN_TRADITIONAL = "https://dictionary.cambridge.org/dictionary/english-chinese-traditional/"
 
 # ----------Request Web Resource----------
-def search_cambridge(con, cur, input_word, is_fresh=False):
-    req_url = get_request_url(CAMBRIDGE_DICT_BASE_URL, input_word, DICTS[0])
+def search_cambridge(con, cur, input_word, is_fresh=False, is_ch=False):
+    if is_ch:
+        req_url = get_request_url(CAMBRIDGE_DICT_BASE_URL_CN, input_word, DICTS[0])
+    else:
+        req_url = get_request_url(CAMBRIDGE_DICT_BASE_URL, input_word, DICTS[0])
 
     if not is_fresh:
         cached = dict.cache_run(con, cur, input_word, req_url, DICTS[0])
         if not cached:
-            fresh_run(con, cur, req_url, input_word)
+            fresh_run(con, cur, req_url, input_word, is_ch)
     else:
-        fresh_run(con, cur, req_url, input_word)
+        fresh_run(con, cur, req_url, input_word, is_ch)
 
 
-def fresh_run(con, cur, req_url, input_word):
-    result = fetch_cambridge(req_url, input_word)
-    found = result[0]
-
-    if found:
-        res_url, res_text = result[1]
-        soup = make_a_soup(res_text)
-        response_word = parse_response_word(soup)
-
-        parse_thread = threading.Thread(target=parse_and_print, args=(res_url, soup))
-        parse_thread.start()
-
-        dict.save(con, cur, input_word, response_word, res_url, res_text)
-
-    else:
-        spell_res_url, spell_res_text = result[1]
-        soup = make_a_soup(spell_res_text)
-        parse_and_print(spell_res_url, soup)
-
-
-def fetch_cambridge(req_url, input_word):
-    """Get response url and response text for future parsing."""
+def fetch_cambridge(req_url, input_word, is_ch):
+    """Get response url and response text for later parsing."""
 
     with requests.Session() as session:
-        session.trust_env = False
+        session.trust_env = False   # not to use proxy
         res = dict.fetch(req_url, session)
 
-        if res.url == CAMBRIDGE_DICT_BASE_URL:
+        if res.url == CAMBRIDGE_DICT_BASE_URL or res.url == CAMBRIDGE_DICT_BASE_URL_CN:
             logger.debug(f'{OP[6]} "{input_word}" in {DICTS[0]}')
+            if is_ch:
+                spell_req_url = get_request_url_spellcheck(CAMBRIDGE_SPELLCHECK_URL_CN, input_word)
+            else:
+                spell_req_url = get_request_url_spellcheck(CAMBRIDGE_SPELLCHECK_URL, input_word)
 
-            spell_req_url = get_request_url_spellcheck(
-                CAMBRIDGE_SPELLCHECK_URL, input_word
-            )
             spell_res = dict.fetch(spell_req_url, session)
             spell_res_url = spell_res.url
             spell_res_text = spell_res.text
-
             return False, (spell_res_url, spell_res_text)
+
         else:
             res_url = parse_response_url(res.url)
             res_text = res.text
@@ -80,30 +69,48 @@ def fetch_cambridge(req_url, input_word):
             return True, (res_url, res_text)
 
 
+def fresh_run(con, cur, req_url, input_word, is_ch):
+    """Print the result without cache."""
+
+    result = fetch_cambridge(req_url, input_word, is_ch)
+    found = result[0]
+
+    if found:
+        res_url, res_text = result[1]
+        soup = make_a_soup(res_text)
+        response_word = parse_response_word(soup)
+
+        first_dict = parse_first_dict(res_url, soup)
+
+        parse_thread = threading.Thread(target=parse_and_print, args=(first_dict, res_url))
+        parse_thread.start()
+        # parse_thread.join()
+
+        dict.save(con, cur, input_word, response_word, res_url, str(first_dict))
+
+    else:
+        spell_res_url, spell_res_text = result[1]
+
+        logger.debug(f"{OP[1]} {spell_res_url}")
+        soup = make_a_soup(spell_res_text)
+        nodes = soup.find("div", "hfl-s lt2b lmt-10 lmb-25 lp-s_r-20")
+        suggestions = []
+
+        for ul in nodes.find_all("ul", "hul-u"):
+            for i in ul.find_all("li"):
+                sug = replace_all(i.text)
+                suggestions.append(sug)
+
+        logger.debug(f"{OP[4]} the parsed result of {spell_res_url}")
+        dict.print_spellcheck(con, cur, input_word, suggestions, DICTS[0], is_ch)
+
+
 # ----------The Entry Point For Parse And Print----------
-def parse_and_print(res_url, soup):
-    """The entry point to parse the dict and print the info about the word."""
 
-    logger.debug(f"{OP[1]} the fetched result of {res_url}")
-
-    if "spellcheck" in res_url:
-        logger.debug(f"{OP[4]} the parsed result of {res_url}")
-        parse_spellcheck(soup)
-        sys.exit()
-
-    blocks, first_dict = parse_dict_blocks(res_url, soup)
+def parse_and_print(first_dict, res_url):
+    """Parse and print different sections for the word."""
 
     logger.debug(f"{OP[4]} the parsed result of {res_url}")
-    for block in blocks:
-        parse_dict_head(block)
-        parse_dict_body(block)
-    parse_dict_name(first_dict)
-
-
-def parse_dict_blocks(res_url, soup):
-    """Parse different form sections for the word."""
-
-    first_dict = parse_first_dict(res_url, soup)
 
     attempt = 0
     while True:
@@ -113,59 +120,59 @@ def parse_dict_blocks(res_url, soup):
             )
         except AttributeError as e:
             attempt = call_on_error(e, res_url, attempt, OP[3])
-
+            continue
         else:
             if result:
                 blocks = first_dict.find_all(
                     "div", ["pr entry-body__el", "entry-body__el clrd js-share-holder"]
                 )
+                break
             else:
                 blocks = first_dict.find_all("div", "pr idiom-block")
+                break
 
-            return blocks, first_dict
+    for block in blocks:
+        parse_dict_head(block)
+        parse_dict_body(block)
+    parse_dict_name(first_dict)
 
 
 def parse_first_dict(res_url, soup):
     """Parse the dict section of the page for the word."""
 
     attempt = 0
+    logger.debug(f"{OP[1]} {res_url}")
+
     while True:
         try:
-            first_dict = soup.find("div", "pr dictionary")
+            # first_dict = soup.find("div", "pr dictionary")
+            first_dict = soup.find("div", "pr di superentry")
         except Exception as e:
             attempt = call_on_error(e, res_url, attempt, OP[3])
+            continue
+        else:
+            if first_dict is None:
+                attempt = call_on_error(ParsedNoneError(res_url), res_url, attempt, OP[3])
+                continue
+            else:
+                break
 
-        if first_dict is None:
-            attempt = call_on_error(ParsedNoneError(), res_url, attempt, OP[3])
-
-        return first_dict
-
-
-def parse_spellcheck(soup):
-    """Parse Cambridge spellcheck page and print it to the terminal."""
-
-    content = soup.find("div", "hfl-s lt2b lmt-10 lmb-25 lp-s_r-20")
-
-    try:
-        title = content.h1.text.split("for")[1].strip()
-    except IndexError:
-        title = content.find_all("h1")[1].text.split("for")[1].strip()
-
-    console.print("[bold yellow]" + title)
-
-    for ul in content.find_all("ul", "hul-u"):
-        notice = ul.find_previous_sibling().text
-        console.print("[bold #3C8DAD]" + "\n" + notice)
-        for i in ul.find_all("li"):
-            suggestion = replace_all(i.text)
-            console.print("[#b2b2b2]" + "  • " + suggestion)
+    return first_dict
 
 
 # ----------Parse Response Word----------
 def parse_response_word(soup):
     "Parse the response word from html head title tag."
 
-    response_word = soup.find("title").text.split("|")[0].strip().lower()
+    temp = soup.find("title").text.split("-")[0].strip()
+    if "|" in temp:
+        response_word = temp.split("|")[0].strip().lower()
+
+    elif "in Simplified Chinese" in temp:
+        response_word = temp.split("in Simplified Chinese")[0].strip().lower()
+    else:
+        response_word = temp.lower()
+
     return response_word
 
 
@@ -237,12 +244,18 @@ def parse_head_domain(head):
 
 
 def parse_head_usage(head):
-    if head.find("span", "lab dlab"):
-        w_usage = replace_all(head.find("span", "lab dlab").text)
-        return w_usage
-    if head.find_next_sibling("span", "lab dlab"):
-        w_usage = replace_all(head.find_next_sibling("span", "lab dlab").text)
+    head_usage = head.find("span", "lab dlab")
+
+    # NOTE: <span class = "var dvar"> </span>, attrs["class"] returns a list 
+    if head_usage is not None and head_usage.parent.attrs["class"] != ["var", "dvar"]:
+        w_usage = replace_all(head_usage.text)
         return w_usage
+
+    head_usage_next = head.find_next_sibling("span", "lab dlab")
+    if head_usage_next is not None and head_usage_next.parent.attrs["class"] != ["var", "dvar"]:
+        w_usage_next= replace_all(head_usage_next.text)
+        return w_usage_next
+
     return ""
 
 
@@ -338,6 +351,12 @@ def parse_meaning(def_block):
         meaning_words = replace_all(meaning_b.text)
         print("\033[33m" + meaning_words + "\033[0m")
 
+    # Print the meaning's specific language translation if any
+    meaning_lan = def_block.find("span", "trans dtrans dtrans-se break-cj")
+    if meaning_lan:
+        meaning_lan_words = meaning_lan.text
+        print("\033[93m" + meaning_lan_words + "\033[0m")
+
 
 def parse_pmeaning(def_block):
     meaning_b = def_block.find("div", "def ddef_d db")
@@ -350,6 +369,12 @@ def parse_pmeaning(def_block):
         meaning_words = replace_all(meaning_b.text)
         print("  " + "\033[33m" + meaning_words + "\033[0m")
 
+    # Print the meaning's specific language translation if any
+    meaning_lan = def_block.find("span", "trans dtrans dtrans-se break-cj")
+    if meaning_lan:
+        meaning_lan_words = meaning_lan.text
+        print("  " + "\033[93m" + meaning_lan_words + "\033[0m")
+
 
 def parse_example(def_block):
     # NOTE:
@@ -363,6 +388,14 @@ def parse_example(def_block):
     for e in def_block.find_all("div", "examp dexamp"):
         if e is not None:
             example = replace_all(e.find("span", "eg deg").text)
+
+            # Print the exmaple's specific language translation if any
+            example_lan = e.find("span", "trans dtrans dtrans-se hdb break-cj")
+            if example_lan is not None:
+                example_lan_sent = example_lan.text
+            else:
+                example_lan_sent = ""
+
             if e.find("span", "lab dlab"):
                 lab = replace_all(e.find("span", "lab dlab").text)
                 console.print(
@@ -373,6 +406,8 @@ def parse_example(def_block):
                     + " "
                     + "[#b2b2b2]"
                     + example
+                    + " "
+                    + example_lan_sent
                 )
             elif e.find("span", "gram dgram"):
                 gram = replace_all(e.find("span", "gram dgram").text)
@@ -384,6 +419,8 @@ def parse_example(def_block):
                     + " "
                     + "[#b2b2b2]"
                     + example
+                    + " "
+                    + exmaple_lan_sent
                 )
             elif e.find("span", "lu dlu"):
                 lu = replace_all(e.find("span", "lu dlu").text)
@@ -395,9 +432,11 @@ def parse_example(def_block):
                     + " "
                     + "[#b2b2b2]"
                     + example
+                    + " "
+                    + example_lan_sent
                 )
             else:
-                console.print("[#b2b2b2]" + "  • " + example)
+                console.print("[#b2b2b2]" + "  • " + example + " " + example_lan_sent)
 
 
 def parse_synonym(def_block):
@@ -558,4 +597,4 @@ def parse_dict_name(first_dict):
     dict_info = replace_all(first_dict.small.text).strip("(").strip(")")
     dict_name = dict_info.split("©")[0]
     dict_name = dict_name.split("the")[-1]
-    console.print(dict_name + "\n", justify="right", style="bold")
+    console.print(dict_name + "\n", justify="right")
